/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_kfs_arc_
#define _h_kfs_arc_

#ifndef _h_kfs_extern_
#include <kfs/extern.h>
#endif

#ifndef _h_klib_defs_
#include <klib/defs.h>
#endif

#include <kfs/toc.h>

#ifdef __cplusplus
extern "C" {
#endif

struct KDirectory;
struct KToc;
struct vector;

/* validate_header_offsets
 *	This function is a special friend function to the archive package used
 *	to validate that the compiler used to build the package matches 
 *	used structure offsets "correctly" to enable the parsing of TAR files.
 * NOTE:
 *	This function should not be used except by ktartest (klib/tools/ktartest.c).
 *	It is built only in a special logging version of the libraries.
 */
KFS_EXTERN bool CC validate_header_offsets(void);

typedef struct KArcDir KArcDir;
typedef struct KArcFile KArcFile;

KFS_EXTERN rc_t CC KDirectoryOpenArcDirRead(const struct KDirectory * self, 
    const struct KDirectory ** pdir, bool chroot, const char * path, KTocFSType baseType,
    rc_t ( CC * parse )(struct KToc *,const void *,
        bool( CC * )(const struct KDirectory*, const char *, void*), void*),
        bool (CC* filter )(const KDirectory*, const char *, void *), void * filterparam);

KFS_EXTERN rc_t CC KArcDirPersistHeader	(const KArcDir * self,
    void ** buffer, size_t * buffer_size, uint64_t * file_sile, uint32_t align,
    rc_t ( CC *sort )(const struct KDirectory*, struct Vector*) );

/* ----------------------------------------------------------------------
 * KArcDirGetTOC
 *
 * [RET] rc_t					0 for success; anything else for a failure
 *						see itf/klib/rc.h for general details
 * [IN]  const KArcDir *	self		Object oriented C; KArcDir object for this method
 * [OUT] const KToc **		toc
 */
KFS_EXTERN rc_t CC KArcDirGetTOC (const KArcDir * self, const struct KToc ** toc);

#ifdef __cplusplus
}
#endif

#endif /* _h_kfs_arc_ */
