/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_kfs_sra_
#define _h_kfs_sra_

#ifndef _h_kfs_extern_
#include <kfs/extern.h>
#endif

#include <klib/defs.h>

#include <stdarg.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef uint32_t KSRAFileAlignment;
enum KSRAFileAlignment
{
    /* align on byte count unit boundaries
     * set this to the number of bytes in the alignment unit
     */
    sraAlignInvalid = 0,
    sraAlign1Byte = 1,
    sraAlign2Byte = 2,
    sraAlign4Byte = 4,
    sraAlign8Byte = 8,
    sraAlign16Byte = 16,
    sraAlign32Byte = 32,
    sraAlign64Byte = 64
};

typedef struct KSraHeader KSraHeader;

KFS_EXTERN rc_t CC SraHeaderMake (KSraHeader ** self, size_t treesize, KSRAFileAlignment alignment);
KFS_EXTERN rc_t CC SraHeaderValidate (const KSraHeader * self, bool * reverse, uint32_t * version);
KFS_EXTERN size_t CC SraHeaderSize (const KSraHeader * self);
#define FS_SRA_CUR_VERSION 1

KFS_EXTERN uint64_t CC SraHeaderGetFileOffset (const KSraHeader * self);

/*--------------------------------------------------------------------------
 * forwards
 */
struct KFile;
struct KDirectory;
struct KArcTOC;

/*--------------------------------------------------------------------------
 * KDirectory
 *  SRA-specific extensions
 */

/* KArcTOCParseSRA
 *	A parse function for sra files suitable for KDirectoryOpenArcDirRead
 *
 * [INOUT] KArcTOC*	self	The TOC (table of contents) to fill in with the parse
 * [IN]    const KFile*	kfile	The opened (as KFile) archive to parse.
 */
KFS_EXTERN rc_t CC KArcTOCParseSRA (struct KArcTOC * self, 
				 const void * kfile,
				 bool(CC*f)(const char * path));


/* KDirectoryOpenSraArchive
 *  opens a named SRA archive
 *
 *  "sra_dir" [ OUT ] - return parameter for archive
 *
 *  "chroot" [ IN ] - if non-zero, the new directory becomes
 *  chroot'd and will interpret paths beginning with '/'
 *  relative to itself.
 *
 *  "path" [ IN ] - NUL terminated string in directory-native
 *  character set denoting sraget archive
 */
KFS_EXTERN int CC KDirectoryOpenSraArchive ( struct KDirectory const *self,
    struct KDirectory const **sra_dir, int chroot, const char *path, ... );
KFS_EXTERN int CC KDirectoryVOpenSraArchive ( struct KDirectory const *self,
    struct KDirectory const **sra_dir, int chroot, const char *path, va_list args );

#ifdef __cplusplus
}
#endif

#endif /* _h_kfs_sra_ */
