/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_klib_status_
#define _h_klib_status_

#ifndef _h_klib_extern_
#include <klib/extern.h>
#endif

#ifndef _h_klib_writer_
#include <klib/writer.h>
#endif

#ifndef WRITER_INLINING
#define WRITER_INLINING 1
#endif

#ifdef __cplusplus
extern "C" {
#endif


/* ----------
 * KStsLevel
 *
 *  KSts levels are numeric and the same higher priority/lower numeric
 *  as with log Level but with no textual equivalents.
 */
typedef int32_t KStsLevel;
enum KStsLevel
{
    ksts_min = -1,
    ksts_off = ksts_min,
    ksts_0,
    ksts_1,
    ksts_2,
    ksts_3,
    ksts_4,
    ksts_5,
    ksts_max = ksts_5
};

KLIB_EXTERN KStsLevel G_sts_level;

KLIB_EXTERN KStsLevel CC KStsLevelGet( void );
KLIB_EXTERN rc_t CC KStsLevelSet( KStsLevel level );
KLIB_EXTERN rc_t CC KStsLevelAdjust( int32_t adjust );

#if WRITER_INLINING
#define KStsLevelGet()      G_sts_level
#define KStsLevelSet(L) (((L>=ksts_min)&&(L<=ksts_max))?((G_sts_level=L),0):RC(rcApp,rcLog,rcUpdating,rcParam,rcOutofrange))
#define KStsLevelAdjust(A) KStsLevelSet(KStatsLevelGet()+A)
#endif


/* -----
 * Handlers for application and library writers.
 */
KLIB_EXTERN KWrtHandler G_sts_handler;
KLIB_EXTERN KWrtHandler G_sts_lib_handler;

KLIB_EXTERN KWrtHandler* CC KStsHandlerGet (void);
KLIB_EXTERN KWrtHandler* CC KStsLibHandlerGet (void);

KLIB_EXTERN KWrtWriter CC KStsWriterGet (void);
KLIB_EXTERN KWrtWriter CC KStsLibWriterGet (void);

KLIB_EXTERN void* CC KStsDataGet (void);
KLIB_EXTERN void* CC KStsLibDataGet (void);

/*
 * Init()
 * Initialize the debug messages module to a known state
 */
KLIB_EXTERN rc_t CC KStsInit (void);


KLIB_EXTERN rc_t CC KStsMsg (const char * fmt, ...);
KLIB_EXTERN rc_t CC KStsLibMsg (const char * fmt, ...);


/* Set
 */
KLIB_EXTERN rc_t CC KStsHandlerSet    (KWrtWriter writer, void * data);
KLIB_EXTERN rc_t CC KStsLibHandlerSet (KWrtWriter writer, void * data);

KLIB_EXTERN rc_t CC KStsHandlerSetStdOut    ();
KLIB_EXTERN rc_t CC KStsLibHandlerSetStdOut ();
KLIB_EXTERN rc_t CC KStsHandlerSetStdErr    ();
KLIB_EXTERN rc_t CC KStsLibHandlerSetStdErr ();

#if WRITER_INLINING

#define KStsHandlerGet()    (&G_sts_handler)
#define KStsLibHandlerGet() (&G_sts_lib_handler)
#define KStsWritererGet()   G_sts_handler.writer
#define KStsLibWriterGet()  G_sts_lib_handler.writer
#define KStsDataGet()       G_sts_handler.data
#define KStsLibDataGet()    G_sts_lib_handler.data

#define KStsHandlerSet(w,d)       ((G_sts_handler.writer=w),(G_sts_handler.data=d),0)
#define KStsLibHandlerSet(w,d)    ((G_sts_lib_handler.writer=w),(G_sts_lib_handler.data=d),0)
#define KStsHandlerSetStdOut()    KStsHandlerSet(DefaultWriter,DefaultWriterStdOut)
#define KStsLibHandlerSetStdOut() KStsLibHandlerSet(DefaultWriter,DefaultWriterStdOut)
#define KStsHandlerSetStdErr()    KStsHandlerSet(DefaultWriter,DefaultWriterStdErr)
#define KStsLibHandlerSetStdErr() KStsLibHandlerSet(DefaultWriter,DefaultWriterStdErr)

#endif

/*
 * A usage could look like
 *
KLIB_EXTERN  * STSMSG (kstatus_2, "Current Status Level is %d\n", KStsLevelGet());
 *
 */
#ifdef _LIBRARY

#define STSMSG(lvl,msg) \
    (void)(((KStsLevelGet() > ksts_off) && (KStsLevelGet() >= lvl) && (KStsLibWriterGet() != NULL)) ? KStsLibMsg msg : 0)

#else

#define STSMSG(lvl,msg) \
    (void)(((KStsLevelGet() > ksts_off) && (KStsLevelGet() >= lvl) && (KStsWriterGet() != NULL)) ? KStsMsg msg : 0)

#endif


#ifdef __cplusplus
}
#endif

#endif /*  _h_klib_status_ */
