/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_log_priv_
#define _h_log_priv_

#ifndef _h_klib_log_
#include <klib/log.h>
#endif

#ifndef _h_klib_rc_
#include <klib/rc.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif



typedef struct KLogDataDefault
{
    int handle;
} KLogDataDefault;

extern KLogDataDefault G_log_data_stdout;
extern KLogDataDefault G_log_data_stderr;
rc_t DefaultLogDriver (void * data, const char * buffer, size_t count);




/* KLogLevelExplain
 *  inserts level description
 */
rc_t KLogLevelExplainInt ( unsigned lvl,
                           char *buffer, size_t bsize, size_t *num_writ );

enum ERCExplain2Options
{
    eRCExOpt_NoMessageIfNoError,
    eRCExOpt_ObjAndStateOnlyIfError,
    eRCExOpt_CompleteMsg
};

#if 0
/* declared in <klib/writer.h> */
rc_t RCExplain2 ( rc_t rc, char *buffer, size_t bsize, size_t *num_writ,
    int rCExplain2Options );
#endif

/*
 * drivers and driver self
 */

typedef struct StreamSelf StreamSelf;
typedef rc_t ( CC * StreamWrite )(StreamSelf * self, const void * data, size_t count, size_t * num_writ);

/* some defaults */

/*--------------------------------------------------------------------------
 * nvp - name/value pair
 */
typedef struct nvp nvp;
struct nvp
{
    const char *name;
    const char *value;
};


/*--------------------------------------------------------------------------
 * Log
 *  global logging
 */

/* LastErrorCodeSet
 *  set the last reported rc_t with log level klogErr or above
 */
void LogLastErrorCodeSet ( KLogLevel lvl, rc_t rc );

/* Timestamp
 *  generates a timestamp string
 */
rc_t LogTimestamp ( char *buffer, size_t bsize, size_t *num_writ );


/* LogSimpleTimestamp
 *  generates a local timestamp string without time zone
 */
rc_t LogSimpleTimestamp ( char *buffer, size_t bsize, size_t *num_writ );


/* PID
 *  generates a process id
 */
rc_t LogPID ( char *buffer, size_t bsize, size_t *num_writ );


/* FlushLine
 *  flushes a line of text to output handler
 */
rc_t KLogFlushLine ( KWrtHandler * handler, char *buffer, size_t bsize, size_t total );


/* InsertSpace
 *  inserts a division after current text
 *
 *  "spacer" [ IN, NULL OKAY ] - optional characters to insert
 */
rc_t LogInsertSpace ( const char *spacer,
    char *buffer, size_t bsize, size_t *num_writ );

/* AppendMsg
 *  appends a message
 */
rc_t LogAppendMsg ( const char *msg,
    char *buffer, size_t bsize, size_t *num_writ );


/* ProduceMsg
 *  given a formatted msg string and parameters,
 *  produce a normalized message string and
 *  parse parameters into name,value pairs
 *
 *  "msg" [ IN ] - formatted message string
 *
 *  "fmt" [ IN ] and "args" [ IN ] - printf-style parameters
 *
 *  "argc" [ OUT ] - return parameter for number of
 *  name,value pairs
 *
 *  "argv" [ OUT ] and "asize" [ IN ] - output array
 *  of nvp where "asize" is size in bytes of "argv"
 *
 *  "nvp_buffer" [ OUT ] and "nvp_size" [ IN ] - buffer for
 *  producing name/value pair data
 *
 *  "offset" [ IN ] - starting offset into buffer
 *  allows existing data in buffer to be passed to
 *  allocation if needed
 *
 *  "buffer" [ OUT ] and "bsize" [ IN ] -
 *  caller-supplied buffer for assembling text
 *
 *  "num_writ" [ OUT ] - number of bytes written to "buffer"
 */
rc_t LogProduceMsg ( const char *msg, const char *fmt, va_list args,
    uint32_t *argc, nvp argv [], size_t asize, char *nvp_buffer, size_t nvp_size,
    size_t offset, char *buffer, size_t bsize, size_t *num_writ );


/* GetRCModuleText
 */
const char *GetRCModuleText ( enum RCModule mod );

/* GetRCTargetText
 */
const char *GetRCTargetText ( enum RCTarget targ );

/* GetRCContextText
 */
const char *GetRCContextText ( enum RCContext ctx );

/* GetRCObjectText
 *  obj can be RCTarget or RCObject
 */
const char *GetRCObjectText ( int obj );

/* GetRCStateText
 */
const char *GetRCStateText ( enum RCState state );


/* GetRCModuleIdxText
 */
const char *GetRCModuleIdxText ( enum RCModule mod );

/* GetRCTargetIdxText
 */
const char *GetRCTargetIdxText ( enum RCTarget targ );

/* GetRCContextIdxText
 */
const char *GetRCContextIdxText ( enum RCContext ctx );

/* GetRCObjectIdxText
 *  obj can be RCTarget or RCObject
 */
const char *GetRCObjectIdxText ( int obj );

/* GetRCStateIdxText
 */
const char *GetRCStateIdxText ( enum RCState state );

/* fix up windows printf shortcomings for integer types 
 * current format is len long at fmt with max max size for the format
 * len is updated to new length if fmt is changed
 */
void print_int_fixup   (char * fmt, size_t * len, size_t max);
void print_float_fixup (char * fmt, size_t * len, size_t max);
void print_char_fixup  (char * fmt, size_t * len, size_t max);


#ifdef __cplusplus
}
#endif

#endif /* _h_log_priv_ */
