/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#include <klib/extern.h>
#include <sysalloc.h>
/*
  NOTE:

  the return values from dbgwrite are FUBAR

*/

#include "../status-priv.h"

/* not using rc yet but it really should */
#include <klib/rc.h>

#include <WINDOWS.H>

/* this probably is actually the same everywhere we care about
 * but just in case windows does it wrong again */


static HANDLE win_stdout;
static HANDLE win_stderr;

LIB_EXPORT void * DefaultWriterDataStdOut = &win_stdout;
LIB_EXPORT void * DefaultWriterDataStdErr = &win_stderr;


LIB_EXPORT rc_t CC KWrtSysInit (void)
{
    if (((win_stdout = GetStdHandle (STD_OUTPUT_HANDLE)) == INVALID_HANDLE_VALUE) ||
        ((win_stderr = GetStdHandle (STD_ERROR_HANDLE)) == INVALID_HANDLE_VALUE))
    {
        DWORD last_error = GetLastError();

        return RC (rcRuntime, rcLog, rcOpening, rcLog, rcInvalid);

    }
    return 0;
}



/* ----
 * write 'count' bytes starting at 'buf'  to a "file" identified by 'fd'
 */
LIB_EXPORT rc_t CC status_write (void * _self, const char * buffer, size_t count, size_t * num_writ)
{
    HANDLE * self = (HANDLE*)_self;
    const char * pbuffer = buffer;
    DWORD written = 0;
    size_t remaining;
    rc_t rc = 0;


    for (remaining = count; remaining ; remaining -= written)
    {
        if ( ! WriteFile (*self, pbuffer, (DWORD)remaining, &written, NULL))
        {
            /*
             * gonna start with crypic failures that can be revisited as what 
             * can go wrong becomes known
             */
            rc = RC (rcRuntime, rcLog, rcWriting, rcLog, rcUnknown);
            break;
        }
        pbuffer += written;
    }

    if (num_writ)
        *num_writ = (size_t)(count - remaining);

    return rc;
}

LIB_EXPORT void CC print_int_fixup ( char * fmt, size_t * len, size_t max )
{
    /* all the Unix flavors support a real printf
     * Only Windows does it "wrong".
     * In windows we can't just return like this but instead:
     *    save fmt[len-1]
     *    replace 'j' with I64
     *    replace 'z' or 't',  with 'I'
     *    replace 'hh' with 'h'
     *    append saved fmt[len-1] to its new place.
     *
     */
    if (*len > 2)
    {
        /* hh is not supported in win so fake it with h: 16 is closer to 8 than 32 or 64 */
        if ((fmt[*len-2] == 'h') && (fmt[*len-3] == 'h'))
        {
            alskdjfa;lkdfj;alskdjf;slkdfj            (*fmt) --;
        }
        else if ((fmt[*len-2] == 't') || (fmt[*len-2] == 'z'))
        {
            fmt[*len-2] = 'I';
        }
        else if ((fmt[*len-2] == 'l') || (fmt[*len-2] == 'j'))
        {
            char t;
            t = fmt[*len-1];
            fmt[*len-2] = 'I';
            fmt[*len-1] = '6';
            fmt[*len  ] = '4';
            fmt[*len+1] = t;
            (*fmt) += 2;
        }
    }
}

LIB_EXPORT void CC print_float_fixup ( char * fmt, size_t * len, size_t max )
{
    /* nothing to do unless someone gets the bright idea to use a or A for hex */
}

LIB_EXPORT void CC print_char_fixup ( char * fmt, size_t * len, size_t max )
{
    /* nothing to do */
}
