/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_writer_priv_
#define _h_writer_priv_

#include <klib/writer.h>
#ifndef _h_klib_rc_
#include <klib/rc.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif


extern size_t wrt_ident_size;
extern char wrt_ident [ 256 ];
extern uint32_t wrt_version;

extern const char *gRCModule_str [];
extern const char *gRCTarget_str [];
extern const char *gRCContext_str [];
extern const char *gRCObject_str [];
extern const char *gRCState_str [];

extern const char *gRCModuleIdx_str [];
extern const char *gRCTargetIdx_str [];
extern const char *gRCContextIdx_str [];
extern const char *gRCObjectIdx_str [];
extern const char *gRCStateIdx_str [];


rc_t KWrtSysInit ( const char * appname, uint32_t vers );



/* KLogLevelExplain
 *  inserts level description
 */
rc_t KLogLevelExplainInt ( unsigned lvl,
                           char *buffer, size_t bsize, size_t *num_writ );

enum ERCExplain2Options
{
    eRCExOpt_NoMessageIfNoError,
    eRCExOpt_ObjAndStateOnlyIfError,
    eRCExOpt_CompleteMsg
};

#if 0
/* declared in <klib/writer.h> */
rc_t RCExplain2 ( rc_t rc, char *buffer, size_t bsize, size_t *num_writ,
    int rCExplain2Options );
#endif

/*--------------------------------------------------------------------------
 * nvp - name/value pair
 */
typedef struct nvp nvp;
struct nvp
{
    const char *name;
    const char *value;
};


/*--------------------------------------------------------------------------
 * Log
 *  global logging
 */
extern size_t log_ident_size;
extern char log_ident [ 64 ];
extern uint32_t log_version;


/* Timestamp
 *  generates a timestamp string
 */
rc_t LogTimestamp ( char *buffer, size_t bsize, size_t *num_writ );

/* PID
 *  generates a process id
 */
rc_t LogPID ( char *buffer, size_t bsize, size_t *num_writ );


/* FlushLine
 *  flushes a line of text to output handler
 */
rc_t KLogFlushLine ( unsigned char *buffer, size_t bsize, size_t total );


/* InsertSpace
 *  inserts a division after current text
 *
 *  "spacer" [ IN, NULL OKAY ] - optional characters to insert
 */
rc_t LogInsertSpace ( const char *spacer,
    char *buffer, size_t bsize, size_t *num_writ );

/* AppendMsg
 *  appends a message
 */
rc_t LogAppendMsg ( const char *msg,
    char *buffer, size_t bsize, size_t *num_writ );


/* ProduceMsg
 *  given a formatted msg string and parameters,
 *  produce a normalized message string and
 *  parse parameters into name,value pairs
 *
 *  "msg" [ IN ] - formatted message string
 *
 *  "fmt" [ IN ] and "args" [ IN ] - printf-style parameters
 *
 *  "argc" [ OUT ] - return parameter for number of
 *  name,value pairs
 *
 *  "argv" [ OUT ] and "asize" [ IN ] - output array
 *  of nvp where "asize" is size in bytes of "argv"
 *
 *  "nvp_buffer" [ OUT ] and "nvp_size" [ IN ] - buffer for
 *  producing name/value pair data
 *
 *  "offset" [ IN ] - starting offset into buffer
 *  allows existing data in buffer to be passed to
 *  allocation if needed
 *
 *  "buffer" [ OUT ] and "bsize" [ IN ] -
 *  caller-supplied buffer for assembling text
 *
 *  "num_writ" [ OUT ] - number of bytes written to "buffer"
 */
rc_t LogProduceMsg ( const char *msg, const char *fmt, va_list args,
    uint32_t *argc, nvp argv [], size_t asize, char *nvp_buffer, size_t nvp_size,
    size_t offset, char *buffer, size_t bsize, size_t *num_writ );


/* GetRCModuleText
 */
const char *GetRCModuleText ( enum RCModule mod );

/* GetRCTargetText
 */
const char *GetRCTargetText ( enum RCTarget targ );

/* GetRCContextText
 */
const char *GetRCContextText ( enum RCContext ctx );

/* GetRCObjectText
 *  obj can be RCTarget or RCObject
 */
const char *GetRCObjectText ( int obj );

/* GetRCStateText
 */
const char *GetRCStateText ( enum RCState state );


/* GetRCModuleIdxText
 */
const char *GetRCModuleIdxText ( enum RCModule mod );

/* GetRCTargetIdxText
 */
const char *GetRCTargetIdxText ( enum RCTarget targ );

/* GetRCContextIdxText
 */
const char *GetRCContextIdxText ( enum RCContext ctx );

/* GetRCObjectIdxText
 *  obj can be RCTarget or RCObject
 */
const char *GetRCObjectIdxText ( int obj );

/* GetRCStateIdxText
 */
const char *GetRCStateIdxText ( enum RCState state );



size_t KWrtFmt_rc_t  (char * buffer, size_t bufsize, const char * fmt, rc_t rc_in);
size_t KWrtFmt_KTime_t (char * buffer, size_t bufsize, const char * fmt_ignored, KTime_t time_in);

extern const char * RCModuleStrings[];
extern const char * RCTargetStrings[];
extern const char * RCContextStrings[];
extern const char * RCObjectStrings[];
extern const char * RCStateStrings[];


/* fix up windows printf shortcomings for integer types 
 * current format is len long at fmt with max max size for the format
 * len is updated to new length if fmt is changed
 */
void print_int_fixup   (char * fmt, size_t * len, size_t max);
void print_float_fixup (char * fmt, size_t * len, size_t max);
void print_char_fixup  (char * fmt, size_t * len, size_t max);


#ifdef __cplusplus
}
#endif

#endif /* _h_writer_priv_ */
