/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#include <sysalloc.h>
#include <vdb/xform.h>
#include <klib/rc.h>

#include <stdlib.h>
#include <assert.h>
#include <string.h>

static
rc_t CC process_position ( void *self, const VXformInfo *info,
    int64_t row_id, const VFixedRowResult *rslt,
    uint32_t argc, const VRowData argv [] )
{
    uint32_t cur, prev;

    uint16_t *dst = rslt -> base;
    const uint16_t *src = argv [ 0 ] . u . data . base;
    uint32_t i, count = argv [ 0 ] . u . data . elem_count;

    dst += rslt -> first_elem;
    src += argv [ 0 ] . u . data . first_elem;

    for ( prev = 0, i = 0; i < count; prev = cur, ++ i )
    {
        cur = src [ i ];
        if ( cur > 0x7FFF )
            cur = ( uint8_t ) cur;
        while ( prev > cur )
            cur += 256;
        dst [ i ] = cur;
    }

    return 0;
}


/* process_position
 *  convert 8 bit integration into 16 bit
 */
VTRANSFACT_IMPL ( NCBI_SRA__454__process_position, 1, 0, 0 ) ( const void *fself, const VXfactInfo *info,
    VFuncDesc *rslt, const VFactoryParams *cp, const VFunctionParams *dp )
{
    rslt -> u . pf = process_position;
    rslt -> variant = vftFixedRow;
    return 0;
}
