/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/
#include <vdb/extern.h>

#include <sra/sradb.h>
#include <vdb/xform.h>
#include <vdb/table.h>
#include <kdb/index.h>
#include <klib/data-buffer.h>
#include <klib/text.h>
#include <klib/rc.h>
#include <sysalloc.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

/*  function utf8 idx:text:insert #1.0 < ascii IDX > ( * utf8 COL );
 *  generates input to skey index. 
 *
 *  an attempt is made to insert COL into indicated text index
 *  ( normally 'skey' ). if the insert succeeds,
 *  then the output for the row is empty.
 *
 *  if the insert fails due to key duplication, an attempt is made to
 *  extend the id range of associated rows. depending upon the type of index,
 *  this may succeed or fail, e.g. if the existing row range for INPUT is
 *  n..m where m = row_id - 1, the range can be extended to n..row_id and
 *  the update succeeds. if the index supports discontiguous id ranges, the
 *  update will also succeed. upon any success updating the index, the output
 *  row will be empty.
 *
 *  finally, if the cannot be inserted into the index
 *  nor the existing id range updated, the output for the row will be COL.
 *
 */

static
rc_t CC idx_text_insert_impl( void *self, const VXformInfo *info, int64_t row_id,
                       VRowResult *rslt, uint32_t argc, const VRowData argv [] )
{
    rc_t rc;
    const char *name;
    uint32_t name_len;
    KIndex *ndx=self;
    char sbuf[4096];

    rslt -> elem_count = 0;

    name = argv [ 0 ] . u . data . base;
    name_len = argv [ 0 ] . u . data . elem_count;
    name += argv [ 0 ] . u . data . first_elem;
    if(name_len < sizeof(sbuf)){
	memcpy(sbuf,name,name_len);
	sbuf[name_len]='\0';
	rslt -> data -> elem_bits = 8;
	rc = KIndexInsertText ( ndx, false, sbuf, row_id );
	if (rc == 0){ /** success, the return row should be empty ***/
	    rslt -> data -> elem_count = 0;
	} else {/* insert failed for whatever reason - return name */
	    KDataBufferWhack(rslt->data);
	    rslt->elem_count = name_len;
	    rc = KDataBufferSub(&argv [ 0 ] . u . data, rslt->data, 0, rslt->elem_count);
	}
    } else {
	rc=RC(rcVDB, rcFunction, rcExecuting, rcBuffer, rcInsufficient);
    }
    return rc;
}

VTRANSFACT_IMPL ( idx_text_insert, 1, 0, 0 ) ( const void *self,
    const VXfactInfo *info, VFuncDesc *rslt, const VFactoryParams *cp, const VFunctionParams *dp )
{
    rc_t rc;
    KIndex *ndx;
    
    rc = VTableCreateIndex ( ( VTable* ) info -> tbl, &ndx, kitText | kitProj, kcmOpen,
                             "%.*s", cp->argv[0].count, cp->argv[0].data.ascii );
    if( rc == 0 ) {
        rslt->self = ndx;
        rslt->whack = ( void ( CC * ) ( void* ) ) KIndexRelease;
        rslt->variant = vftRow;
        rslt->u.rf = idx_text_insert_impl;
    }    
    return rc;
}
