/*===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

#if ENCODING
static rc_t ENCODE(szbuf *scratch, uint8_t dst[], size_t dsize, size_t *used,
                   int64_t *Min, uint8_t *planes, const STYPE Y[], unsigned N)
{
    unsigned i;
    size_t j;
    unsigned k;
    rc_t rc;
    STYPE min;
    STYPE max;
    uint64_t mask;
    unsigned m;
    
    *used = 0;
    
    for (i = 1, max = min = Y[0]; i < N; ++i) {
        if (min > Y[i])
            min = Y[i];
        if (max < Y[i])
            max = Y[i];
    }
    *Min = min;
    *planes = 0;
    
    for (j = k = 0, m = 1, mask = 0xFF; k < 64; k += 8, m <<= 1, mask <<= 8) {
        szbuf s2;
        
        for (i = 0; i != N; ++i) {
            uint64_t val = Y[i] - min;
            
            if ((((uint8_t *)scratch->buf)[i] = (uint8_t)((val & mask) >> k)) != 0)
                *planes |= m;
        }
        if (*planes & m) {
            s2.used = 0;
            s2.size = dsize - j;
            s2.buf  = dst + j;
            
            rc = zlib_compress(&s2, scratch->buf, N,
                               Z_DEFAULT_STRATEGY, Z_BEST_COMPRESSION);
            if (rc) return rc;
            
            j += s2.used;
        }
    }
    *used = j;
    return 0;
}
#endif

#if DECODING
static rc_t DECODE(uint8_t *scratch, STYPE Y[], unsigned N, int64_t Min,
                   uint8_t planes, const uint8_t src[], size_t ssize)
{
    unsigned k;
    size_t j;
    unsigned m;
    unsigned i;
    rc_t rc;
    STYPE min = (STYPE)Min;
    
    memset(Y, 0, sizeof(Y[0]) * N);
    for (j = k = 0, m = 1; m < 0x100; m <<= 1, k += 8) {
        size_t n;
        
        if ((planes & m) == 0)
            continue;
        
        rc = zlib_decompress(scratch, N, &n, src + j, ssize - j);
        if (rc) return rc;
        j += n;
        
        for (i = 0; i != N; ++i)
            Y[i] |= scratch[i] << k;
    }
    for (i = 0; i != N; ++i)
        Y[i] += min;
    return 0;
}
#endif
