/*===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */
#include <vdb/extern.h>
#include <klib/defs.h>
#include <klib/rc.h>
#include <vdb/xform.h>
#include <vdb/schema.h>
#include <sysalloc.h>

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

typedef struct self_t self_t;
struct self_t {
    rc_t (*f)(const self_t *self, void *Dst, const void *Src, const void *In, uint64_t elem_count);
    uint32_t elem_count;
    void *from;
    void *to;
};

static
void CC whack_self ( void *data )
{
    self_t *self = data;
    free ( self -> from );
    free ( self );
}

#define A_TYPE(SIZE) uint ## SIZE ##_t
#define B_TYPE(SIZE) uint ## SIZE ##_t

#define LSEARCH(B_SIZE, A_SIZE) lsearch_ ## B_SIZE ## _ ## A_SIZE
#define LSEARCH_RTC(B_SIZE, A_SIZE) lsearch_rtc_ ## B_SIZE ## _ ## A_SIZE
#define MAKE_LSEARCH(B_SIZE, A_SIZE) make_lsearch_ ## B_SIZE ## _ ## A_SIZE
#define DEF_lsearch(A_SIZE, B_SIZE) \
static rc_t LSEARCH(A_SIZE, B_SIZE) (const self_t *self, void *Dst, const void *In, const void *Src, uint64_t elem_count) { \
    uint64_t i;                                                                          \
    B_TYPE(B_SIZE) *dst = Dst;                                                      \
    const B_TYPE(B_SIZE) *src = Src;                                                \
    const A_TYPE(A_SIZE) *in = In;                                                  \
    const A_TYPE(A_SIZE) *from = self->from;                                        \
    const B_TYPE(B_SIZE) *to = self->to;                                            \
                                                                                    \
    for (i = 0; i != elem_count; ++i) {                                           \
        uint32_t j;                                                                      \
        uint32_t n = self->elem_count;                                                   \
        B_TYPE(B_SIZE) src_val = src[i];                                            \
        const A_TYPE(A_SIZE) cmp_val = in[i];                                       \
                                                                                    \
        for (j = 0; j != n; ++j) {                                                  \
            if (cmp_val == from[j]) {                                               \
                src_val = to[j];                                                    \
                break;                                                              \
            }                                                                       \
        }                                                                           \
        dst[i] = src_val;                                                           \
    }                                                                               \
    return 0;                                                                       \
}                                                                                   \
                                                                                    \
static rc_t LSEARCH_RTC(A_SIZE, B_SIZE) (const self_t *self, void *Dst, const void *In, const void *Src, uint64_t elem_count) { \
    uint64_t i;                                                                          \
    B_TYPE(B_SIZE) *dst = Dst;                                                      \
    const A_TYPE(A_SIZE) *in = In;                                                  \
    const A_TYPE(A_SIZE) *from = self->from;                                        \
    const B_TYPE(B_SIZE) *to = self->to;                                            \
                                                                                    \
    for (i = 0; i != elem_count; ++i) {                                           \
        uint32_t j;                                                                      \
        uint32_t n = self->elem_count;                                                   \
        const A_TYPE(A_SIZE) cmp_val = in[i];                                       \
                                                                                    \
        for (j = 0; j != n; ++j) {                                                  \
            if (cmp_val == from[j])                                                 \
                goto FOUND;                                                         \
        }                                                                           \
        return RC(rcVDB, rcFunction, rcExecuting, rcConstraint, rcViolated);        \
FOUND:                                                                              \
        dst[i] = to[j];                                                             \
    }                                                                               \
    return 0;                                                                       \
}                                                                                   \
                                                                                    \
static rc_t MAKE_LSEARCH(A_SIZE, B_SIZE) (self_t *self, const VFactoryParams *p) {  \
    A_TYPE(A_SIZE) *from;                                                           \
    B_TYPE(B_SIZE) *to;                                                             \
                                                                                    \
    self->elem_count = p->argv[0].count;                                            \
    from = malloc(self->elem_count * (sizeof(from[0]) + sizeof(to[0])));            \
    if (from == NULL)                                                               \
        return RC(rcVDB, rcFunction, rcConstructing, rcMemory, rcExhausted);        \
                                                                                    \
    to = (B_TYPE(B_SIZE) *)(from + self->elem_count);                               \
    memcpy(from, p->argv[0].data.u8, self->elem_count * sizeof(from[0]));           \
    memcpy(to, p->argv[1].data.u8, self->elem_count * sizeof(to[0]));               \
    self->from = from;                                                              \
    self->to = to;                                                                  \
    return 0;                                                                       \
}

#define LOOKUP(SIZE) lookup_ ## SIZE
#define LOOKUP_RTC(SIZE) lookup_rtc_ ## SIZE
#define MAKE_LOOKUP(SIZE) make_lookup_ ## SIZE
#define DEF_lookup(SIZE) \
static rc_t LOOKUP(SIZE)(const self_t *self, void *Dst, const void *In, const void *Src, uint64_t elem_count) { \
    uint64_t i;                                                                          \
    B_TYPE(SIZE) *dst = Dst;                                                        \
    const B_TYPE(SIZE) *src = Src;                                                  \
    const uint8_t *in = In;                                                         \
    const B_TYPE(SIZE) *to = self->to;                                              \
    const uint8_t *mask = self->from;                                               \
                                                                                    \
    for (i = 0; i != elem_count; ++i) {                                             \
        uint8_t j = in[i];                                                          \
                                                                                    \
        if (mask[j])                                                                \
            dst[i] = to[j];                                                         \
        else                                                                        \
            dst[i] = src[i];                                                        \
    }                                                                               \
    return 0;                                                                       \
}                                                                                   \
                                                                                    \
static rc_t LOOKUP_RTC(SIZE)(const self_t *self, void *Dst, const void *In, const void *Src, uint64_t elem_count) { \
    uint64_t i;                                                                          \
    B_TYPE(SIZE) *dst = Dst;                                                        \
    const uint8_t *in = In;                                                         \
    const B_TYPE(SIZE) *to = self->to;                                              \
    const uint8_t *mask = self->from;                                               \
                                                                                    \
    for (i = 0; i != elem_count; ++i) {                                             \
        uint8_t j = in[i];                                                          \
                                                                                    \
        if (mask[j])                                                                \
            dst[i] = to[j];                                                         \
        else                                                                        \
            return RC(rcVDB, rcFunction, rcExecuting, rcConstraint, rcViolated);    \
    }                                                                               \
    return 0;                                                                       \
}                                                                                   \
                                                                                    \
static rc_t MAKE_LOOKUP(SIZE)(self_t *self, const VFactoryParams *p) {              \
    uint32_t i;                                                                          \
    B_TYPE(SIZE) *to;                                                               \
    uint8_t *mask;                                                                  \
                                                                                    \
    mask = calloc(256, sizeof(to[0]) + sizeof(mask[0]));                            \
    if (mask == NULL)                                                               \
        return RC(rcVDB, rcFunction, rcConstructing, rcMemory, rcExhausted);        \
                                                                                    \
    to = (B_TYPE(SIZE) *)&mask[256];                                                \
    for (i = 0; i != p->argv[0].count; ++i) {                                       \
        uint8_t j = p->argv[0].data.u8[i];                                          \
                                                                                    \
        mask[j] = -1;                                                               \
        to[j] = ((const B_TYPE(SIZE) *)p->argv[1].data.u8)[i];                      \
    }                                                                               \
    self->from = mask;                                                              \
    self->to = to;                                                                  \
    self->elem_count = 256;                                                         \
    return 0;                                                                       \
}

DEF_lookup(8)
DEF_lookup(16)
DEF_lookup(32)
DEF_lookup(64)

DEF_lsearch(16, 8)
DEF_lsearch(16, 16)
DEF_lsearch(16, 32)
DEF_lsearch(16, 64)

DEF_lsearch(32, 8)
DEF_lsearch(32, 16)
DEF_lsearch(32, 32)
DEF_lsearch(32, 64)

DEF_lsearch(64, 8)
DEF_lsearch(64, 16)
DEF_lsearch(64, 32)
DEF_lsearch(64, 64)

static
rc_t CC array_func(
                void *Self,
                const VXformInfo *info,
                void *Dst,
                const void *Src,
                uint64_t elem_count
) {
    const self_t *self = Self;
    return self->f(self, Dst, Src, Src, elem_count);
}

static
rc_t CC fixed_row_func(
                    void *Self,
                    const VXformInfo *info,
                    int64_t row_id,
                    const VFixedRowResult *rslt,
                    uint32_t argc,
                    const VRowData argv[]
) {
    const self_t *self = Self;

    /* base pointers to pages */
    uint8_t *dst = rslt -> base;
    const uint8_t *in = argv [ 0 ] . u . data . base;
    const uint8_t *src = argv [ 1 ] . u . data . base;

    /* offset by bytes to first element of row */
    dst += ( rslt -> elem_bits * rslt -> first_elem ) >> 3;
    in += ( argv [ 0 ] . u . data . elem_bits * argv [ 0 ] . u . data . first_elem ) >> 3;
    src += ( argv [ 1 ] . u . data . elem_bits * argv [ 1 ] . u . data . first_elem ) >> 3;

    /* invoke function */
    return self -> f ( self, dst, in, src, rslt -> elem_count );
}

static bool check_type(const VTypedesc *desc) {
    switch (desc->domain) {
    case vtdUint:
    case vtdInt:
        switch (desc->intrinsic_bits) {
        case 8:
        case 16:
        case 32:
        case 64:
            break;
        default:
            return false;
        }
        break;
    case vtdFloat:
        switch (desc->intrinsic_bits) {
        case 32:
        case 64:
            break;
        default:
            return false;
        }
        break;
    case vtdAscii:
        switch (desc->intrinsic_bits) {
        case 8:
            break;
        default:
            return false;
        }
        break;
    default:
        return false;
    }
    return true;
}

struct impl_func_t {
    rc_t (*search)(const self_t *self, void *Dst, const void *In, const void *Src, uint64_t elem_count);
    rc_t (*make_table)(self_t *self, const VFactoryParams *p);
};

static struct impl_func_t find_impl(int a_size, int b_size, int elem_count, int run_time_check) {
    struct impl_func_t y;
	
    switch (a_size) {
    case 8:
        switch (b_size) {
        case 8:
            y.search = run_time_check ? LOOKUP_RTC(8) : LOOKUP(8);
            y.make_table = MAKE_LOOKUP(8);
            break;
        case 16:
            y.search = run_time_check ? LOOKUP_RTC(16) : LOOKUP(16);
            y.make_table = MAKE_LOOKUP(16);
            break;
        case 32:
            y.search = run_time_check ? LOOKUP_RTC(32) : LOOKUP(32);
            y.make_table = MAKE_LOOKUP(32);
            break;
        case 64:
            y.search = run_time_check ? LOOKUP_RTC(64) : LOOKUP(64);
            y.make_table = MAKE_LOOKUP(64);
            break;
        default:
            /* tell compiler we know about bit sizes */
            y.search = NULL;
            y . make_table = NULL;
        }
        break;
    case 16:
        switch (b_size) {
        case 8:
            y.search = run_time_check ? LSEARCH_RTC(16, 8) : LSEARCH(16, 8);
            y.make_table = MAKE_LSEARCH(16, 8);
            break;
        case 16:
            y.search = run_time_check ? LSEARCH_RTC(16, 16) : LSEARCH(16, 16);
            y.make_table = MAKE_LSEARCH(16, 16);
            break;
        case 32:
            y.search = run_time_check ? LSEARCH_RTC(16, 32) : LSEARCH(16, 32);
            y.make_table = MAKE_LSEARCH(16, 32);
            break;
        case 64:
            y.search = run_time_check ? LSEARCH_RTC(16, 64) : LSEARCH(16, 64);
            y.make_table = MAKE_LSEARCH(16, 64);
            break;
        default:
            /* tell compiler we know about bit sizes */
            y.search = NULL;
            y . make_table = NULL;
        }
        break;
    case 32:
        switch (b_size) {
        case 8:
            y.search = run_time_check ? LSEARCH_RTC(32, 8) : LSEARCH(32, 8);
            y.make_table = MAKE_LSEARCH(32, 8);
            break;
        case 16:
            y.search = run_time_check ? LSEARCH_RTC(32, 16) : LSEARCH(32, 16);
            y.make_table = MAKE_LSEARCH(32, 16);
            break;
        case 32:
            y.search = run_time_check ? LSEARCH_RTC(32, 32) : LSEARCH(32, 32);
            y.make_table = MAKE_LSEARCH(32, 32);
            break;
        case 64:
            y.search = run_time_check ? LSEARCH_RTC(32, 64) : LSEARCH(32, 64);
            y.make_table = MAKE_LSEARCH(32, 64);
            break;
        default:
            /* tell compiler we know about bit sizes */
            y.search = NULL;
            y . make_table = NULL;
        }
        break;
    case 64:
        switch (b_size) {
        case 8:
            y.search = run_time_check ? LSEARCH_RTC(64, 8) : LSEARCH(64, 8);
            y.make_table = MAKE_LSEARCH(64, 8);
            break;
        case 16:
            y.search = run_time_check ? LSEARCH_RTC(64, 16) : LSEARCH(64, 16);
            y.make_table = MAKE_LSEARCH(64, 16);
            break;
        case 32:
            y.search = run_time_check ? LSEARCH_RTC(64, 32) : LSEARCH(64, 32);
            y.make_table = MAKE_LSEARCH(64, 32);
            break;
        case 64:
            y.search = run_time_check ? LSEARCH_RTC(64, 64) : LSEARCH(64, 64);
            y.make_table = MAKE_LSEARCH(64, 64);
            break;
        default:
            /* tell compiler we know about bit sizes */
            y.search = NULL;
            y . make_table = NULL;
        }
        break;

    default:
        /* tell compiler we know about bit sizes */
        y.search = NULL;
        y . make_table = NULL;
    }
    return y;
}

/* 
 function < type A, type B > B map #1.0 < A from, B to > ( A in, * B src )
 */
VTRANSFACT_IMPL(vdb_map, 1, 0, 0) (const void *Self, const VXfactInfo *info, VFuncDesc *rslt, const VFactoryParams *cp, const VFunctionParams *dp )
{
    self_t *self;
    struct impl_func_t impl;
    rc_t rc;
	
    if (cp->argv[0].count != cp->argv[1].count)
        return RC(rcVDB, rcFunction, rcConstructing, rcConstraint, rcViolated);
    switch (dp->argc) {
    case 2:
        if (!check_type(&cp->argv[1].desc))
            return RC(rcVDB, rcFunction, rcConstructing, rcType, rcInvalid);
        /* no-break; */
    case 1:
        if (!check_type(&cp->argv[0].desc))
            return RC(rcVDB, rcFunction, rcConstructing, rcType, rcInvalid);
        break;
    default:
        return RC(rcVDB, rcFunction, rcConstructing, rcParam, rcInvalid);
    }
    
    if (dp->argc == 1 && !VTypedeclCommonAncestor(&info->fdesc.fd.td, info->schema, &dp->argv[0].fd.td, 0, 0))
        impl = find_impl(cp->argv[0].desc.intrinsic_bits, cp->argv[1].desc.intrinsic_bits, cp->argv[0].count, 1);
    else
        impl = find_impl(cp->argv[0].desc.intrinsic_bits, cp->argv[1].desc.intrinsic_bits, cp->argv[0].count, 0);
    
    self = malloc(sizeof(self_t));
    if (self == NULL)
        return RC(rcVDB, rcFunction, rcConstructing, rcMemory, rcExhausted);
	
    rc = impl.make_table(self, cp);
    self->f = impl.search;
    
    rslt->self = self;
    rslt->whack = whack_self;
	
    if (dp->argc == 1) {
        rslt->variant = vftArray;
        rslt->u.af = array_func;
    }
    else {
        rslt->variant = vftFixedRow;
        rslt->u.pf = fixed_row_func;
    }
    
    return 0;
}
