/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/
#ifndef _h_tools_dump_files
#define _h_tools_dump_files

#include <klib/rc.h>
#include <kfs/file.h>

typedef struct SRADumperFiles SRADumperFiles;

typedef struct SRADumperFiles_hooks_struct {
    /* called when file just opened, it is ok to call SRADumperFilesWrite in this handler */
    rc_t (*post_open)(const char* key, const SRADumperFiles* files);
    /* called before file is going to be closed */
    void (*pre_close)(const char* key, KFile* file);
} SRADumperFiles_hooks;

/**
  * Make creates initial object and sets file name prefix and extionsion.
  * prefix [IN | NULL] - usually accession is used here.
  * suffix [IN | NULL] - ".fastq", ".sff", ".txt", etc.
  * hooks [IN | NULL]  - set of callback function to add custom handling to file transaction.
  * path, ... [IN]     - directory path for files, if NULL "." is used
  */
rc_t SRADumperFilesMake(const SRADumperFiles** self, const char* prefix, const char* suffix, bool key_as_dir,
                        const SRADumperFiles_hooks* hooks, const char* path, ...);

/**
  * Write buffer to file by key. If file do not exist it is created as "$prefix$key$suffix"
  * key [IN]    - file key. Can be NULL or empty string
  * buffer [IN] - data to write
  * size [IN]   - size of data in buffer
  */
rc_t SRADumperFilesWrite(const SRADumperFiles* self, const char* key, const void* buffer, size_t size);

/**
  * Closes all opened files and optionaly removes empty files
  */
rc_t SRADumperFilesRelease(const SRADumperFiles* self, bool deleteEmpty);

#endif /* _h_tools_dump_files */
